﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Infragistics.Windows.DataPresenter;
using System.Diagnostics;

namespace FieldSpanningByUser
{
    /// <summary>
    /// Interaction logic for Window1.xaml
    /// </summary>
    public partial class Window1 : Window
    {
        // Holds the fields that are above of below the current field
        List<Field> affectedFields = new List<Field>();
        LabelPresenter currentLabelPresenter = null;

        public Window1()
        {
            InitializeComponent();
            
            //wire events
            xamDataGrid1.Loaded += new RoutedEventHandler(xamDataGrid1_Loaded);
            this.PreviewMouseLeftButtonDown+=new MouseButtonEventHandler(xamDataGrid1_PreviewMouseLeftButtonDown);
        }

        void xamDataGrid1_Loaded(object sender, RoutedEventArgs e)
        {
            // register SizeChanged after the XamDataGrid Loads to avoid unnecessary hangling of this event when Fields are generated
            EventManager.RegisterClassHandler(typeof(LabelPresenter), LabelPresenter.SizeChangedEvent, new SizeChangedEventHandler(labelPresenter_SizeChanged));
        }

        void xamDataGrid1_PreviewMouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            DependencyObject source = e.OriginalSource as DependencyObject;

            // LabelPresenter which is currently being spanned
            LabelPresenter lb = Infragistics.Windows.Utilities.GetAncestorFromType(source, typeof(LabelPresenter), false) as LabelPresenter;

            if (lb != null)
            {
                currentLabelPresenter = lb;
                affectedFields.Clear();
                // Get the fields that are below or above the currently spanned field
                affectedFields.AddRange(from field in xamDataGrid1.FieldLayouts[0].Fields
                                        where field.ActualPosition.Column == lb.Field.ActualPosition.ColumnSpan-1
                                        select field);

                // For Debugging purposes
                foreach (Field s in affectedFields)
                    Debug.WriteLine(s.Name);
            }
        }

        void labelPresenter_SizeChanged(object sender, SizeChangedEventArgs e)
        {
            //current label presenter
            LabelPresenter labelPresenter = e.OriginalSource as LabelPresenter;
            //current field for the label presenter
            Field f = labelPresenter.Field;

            //initialize current field's positioning
            int row = f.Row;
            int column = f.Column;
            int rowS = f.RowSpan;
            int colS = f.ColumnSpan;

            if (Keyboard.IsKeyDown(Key.LeftShift) && labelPresenter == currentLabelPresenter)
            {
                #region Increment ColumnSpan
                if (e.NewSize.Width > e.PreviousSize.Width)
                {
                    f.Row = row;
                    f.RowSpan = rowS;
                    f.Column = column;
                    f.ColumnSpan = colS + 1;
                }
                #endregion
                #region Decrement ColumnSpan
                else
                {
                    f.Row = row;
                    f.RowSpan = rowS;
                    f.Column = column;
                    //Column Span cannot be 0
                    if (f.ColumnSpan > 1)
                    {
                        f.ColumnSpan = colS - 1;
                    }
                }
                #endregion
            }
            else
            {
                //Normal resizing
            }
           
            // See if the Field is in the logical column being spanned
            if (affectedFields.Contains(f))
            {
                // if the field is being spanned and not resized - restore width
                f.Settings.LabelWidth = e.PreviousSize.Width;
                affectedFields.Remove(f);
            }
            
            // clear the CurrentLabelPresenter as it is only for the Field which is being Spanned in the PreviewMouseLeftButtonDownEvent
            currentLabelPresenter = null;
        }
    }
}
